/* Copyright (C) 2003 Juan Lang
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 2 or later
 * (the "GPL").
 */
#ifndef __IFDETECT_H__
#define __IFDETECT_H__

#ifndef ETH_ALEN
#define ETH_ALEN 6
#endif

/* Represents an interface.  name is the interface's name, mac is its MAC
 * address, ip its IP address, bcastAddr the broadcast address associated with
 * it, and serverAddr the DHCP server address which granted the IP address if
 * known (and INADDR_NONE otherwise).
 * sock is for your use.
 */
struct InterfaceInfo
{
  char *name;
  u_char mac[ETH_ALEN];
  u_int ip;
  u_int bcastAddr;
  u_int serverAddr;
  int sock;
  struct InterfaceInfo *next;
};

#ifdef __cplusplus
extern "C" {
#endif

/* Returns linked list of InterfaceInfo's describing each DHCP interface.
 * You must do a deep free of the returned array; using freeInterfaceList is
 * the better way to do it.
 * A bit of a misnomer under UNIX, this function finds all the broadcast
 * Ethernet interfaces on your system.  (It can't tell whether they were
 * configured using DHCP.  I've written a shell script that does this; use it
 * in conjunction with this.)
 */
struct InterfaceInfo *findDHCPInterfaces(void);

/* Frees a list created by findDHCPInterfaces.
 */
void freeInterfaceList(struct InterfaceInfo *list);

#ifdef __cplusplus
}
#endif

#endif /* ndef __IFDETECT_H__ */
