/* Copyright (C) 2003 Juan Lang
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 2 or later
 * (the "GPL").
 */
#include <assert.h>
#include <stdio.h>
#include <windows.h>
#include <tchar.h>
#include <iphlpapi.h>
#include "iputil.h"
#include "ifdetect.h"

typedef DWORD (WINAPI *GetAdaptersInfoFunc)(PIP_ADAPTER_INFO, PULONG);
typedef DWORD (WINAPI *GetInterfaceInfoFunc)(PIP_INTERFACE_INFO, PULONG);

HMODULE gIPHelper = NULL;
GetAdaptersInfoFunc gGetAdaptersInfo = NULL;
GetInterfaceInfoFunc gGetInterfaceInfo = NULL;

static void initLibrary(void)
{
  if (gIPHelper == NULL)
    gIPHelper = LoadLibrary(_T("iphlpapi.dll"));
  if (gIPHelper != NULL)
  {
    gGetAdaptersInfo = (GetAdaptersInfoFunc)GetProcAddress(gIPHelper,
     _T("GetAdaptersInfo"));
    gGetInterfaceInfo = (GetInterfaceInfoFunc)GetProcAddress(gIPHelper,
     _T("GetInterfaceInfo"));
  }
}

static void freeLibrary(void)
{
  if (gIPHelper != NULL)
  {
    FreeLibrary(gIPHelper);
    gIPHelper = NULL;
    gGetAdaptersInfo = NULL;
    gGetInterfaceInfo = NULL;
  }
}

/* Finds the adapter with index index in map, and returns a NULL-terminated
 * string representing its name, or NULL if index could not be found.
 * Assumes map is not NULL.
 */
static char *mapIndexToName(ULONG index, PIP_INTERFACE_INFO map)
{
  char *ret = NULL;
  ULONG i;

  assert(map != NULL);
  if (map == NULL)
    return NULL;

  for (i = 0; i < map->NumAdapters; i++)
  {
    if (map->Adapter[i].Index == index)
    {
      OSVERSIONINFO info;

      memset(&info, 0, sizeof(info));
      info.dwOSVersionInfoSize = sizeof(info);
      GetVersionEx(&info);
      if (info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
        ret = strdup((char *)map->Adapter[i].Name);
      else
      {
        int len = wcslen(map->Adapter[i].Name) + 1, j;

        ret = (char *)malloc(len);
        for (j = 0; j < len; j++)
          ret[j] = (char)map->Adapter[i].Name[j];
      }
    }
  }
  return ret;
}

struct InterfaceInfo *findDHCPInterfaces(void)
{
  struct InterfaceInfo *interfaces = NULL;

  initLibrary();
  if (gGetAdaptersInfo != NULL && gGetInterfaceInfo != NULL)
  {
    ULONG adaptersLen = 0;
    PIP_ADAPTER_INFO adapters = NULL;

    gGetAdaptersInfo(NULL, &adaptersLen);
    adapters = (PIP_ADAPTER_INFO)malloc(adaptersLen);
    if (gGetAdaptersInfo(adapters, &adaptersLen) == ERROR_SUCCESS)
    {
      PIP_INTERFACE_INFO map = NULL;
      ULONG mapLen = 0;

      gGetInterfaceInfo(NULL, &mapLen);
      map = (PIP_INTERFACE_INFO)malloc(mapLen);
      if (gGetInterfaceInfo(map, &mapLen) == ERROR_SUCCESS)
      {
        PIP_ADAPTER_INFO ptr = NULL;

        for (ptr = adapters; ptr != NULL; ptr = ptr->Next)
        {
          if (ptr->DhcpEnabled)
          {
            struct InterfaceInfo *newIF = (struct InterfaceInfo *)calloc(1,
             sizeof(struct InterfaceInfo));
            u_int mask;

            newIF->name = mapIndexToName(ptr->Index, map);
            memcpy(newIF->mac, ptr->Address, ETH_ALEN);
            newIF->ip = convertAddr(ptr->IpAddressList.IpAddress.String);
            mask = convertAddr(ptr->IpAddressList.IpMask.String);
            newIF->bcastAddr = (newIF->ip & mask) | ~mask;
            newIF->serverAddr = convertAddr(ptr->DhcpServer.IpAddress.String);
            newIF->next = interfaces;
            interfaces = newIF;
          }
        }
      }
    }
    free(adapters);
  }
  freeLibrary();
  return interfaces;
}

#if 0
int main(void)
{
  struct timeval tv, end;
  struct InterfaceInfo *interfaces, *ptr;

  printf("Initializing time delta to one second from now\n");
  initTimeDelta(&end, 1);
  tv.tv_sec = 1;
  tv.tv_usec = 0;
  interfaces = findDHCPInterfaces();
  for (ptr = interfaces; ptr != NULL; ptr = ptr->next)
  {
    struct in_addr addr;

    addr.s_addr = ptr->ip;
    printf(
     "See adapter %s (mac %02x:%02x:%02x:%02x:%02x:%02x ip %s)\n", ptr->name,
     ptr->mac[0], ptr->mac[1], ptr->mac[2], ptr->mac[3], ptr->mac[4],
     ptr->mac[5], addrToStr(addr));
    addr.s_addr = ptr->serverAddr;
    printf("Its DHCP server is %s\n", addrToStr(addr));
  }
  freeInterfaceList(interfaces);
  adjustTimeDelta(&tv, &end);
  printf("Now %d seconds, %d microseconds remaining\n", tv.tv_sec, tv.tv_usec);
}
#endif
