#include <stdio.h>
#include <windows.h>
#include <tchar.h>
#include <WinDns.h>
#include "strcompat.h"
#include "resquery.h"

typedef DNS_STATUS (WINAPI *DnsQueryAFunc)(PCSTR, WORD, DWORD, PIP4_ARRAY,
 PDNS_RECORD *, PVOID *);
typedef void (WINAPI *DnsRecordListFreeFunc)(PDNS_RECORD, DNS_FREE_TYPE);

HMODULE gDNSApi = NULL;
DnsQueryAFunc gDnsQueryA = NULL;
DnsRecordListFreeFunc gDnsRecordListFree = NULL;

/* Try to load on very OS version; it'll just fail if it can't load, and if MS
 * decides to add this library in a service pack to some OS where it isn't
 * supported yet, it'll just work.
 */
static void initLibrary(void)
{
  if (gDNSApi == NULL)
    gDNSApi = LoadLibrary(_T("dnsapi.dll"));
  if (gDNSApi != NULL)
  {
    gDnsQueryA = (DnsQueryAFunc)GetProcAddress(gDNSApi, _T("DnsQuery_A"));
    gDnsRecordListFree = (DnsRecordListFreeFunc)GetProcAddress(gDNSApi,
     _T("DnsRecordListFree"));
  }
}

static void freeLibrary(void)
{
  if (gDNSApi != NULL)
  {
    FreeLibrary(gDNSApi);
    gDNSApi = NULL;
    gDnsQueryA = NULL;
    gDnsRecordListFree = NULL;
  }
}

int queryWithType(const char *prefix, const char *domain, int type)
{
  initLibrary();
  if (gDnsQueryA != NULL)
  {
    PDNS_RECORD set = NULL;
    DNS_STATUS result;
    WORD wType = type;
    char name[MAX_DOMAIN_NAME], *dotPtr;

    if ((dotPtr = strrchr(prefix, '.')) != NULL && *(dotPtr + 1) != '\0')
    {
      if (domain != NULL && *domain != '\0')
        snprintf(name, sizeof(name), "%s.%s", prefix, domain);
      else
        snprintf(name, sizeof(name), "%s", prefix);
    }
    else
      snprintf(name, sizeof(name), "%s", prefix);
    result = gDnsQueryA(name, wType, DNS_QUERY_TREAT_AS_FQDN, NULL, &set,
     NULL);

    if (result == 0)
    {
      PDNS_RECORD ptr;

      for (ptr = set; ptr != NULL; ptr = ptr->pNext)
      {
        if (ptr == set)
        {
          /* only print name first time around, assume it's the same for
           * each record */
          printf("Name %s\n", ptr->pName);
        }
        if (ptr->wType == DNS_TYPE_SRV)
        {
          DNS_SRV_DATA *srv = &ptr->Data.Srv;

          printf("Target %d %d %d %s\n", srv->wPriority, srv->wWeight,
           srv->wPort, srv->pNameTarget);
        }
      }
      if (gDnsRecordListFree != NULL)
        gDnsRecordListFree(set, DnsFreeRecordList);
    }
  }
  freeLibrary();
  return 0;
}
