#include <assert.h>
#include <stdlib.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include "iptypes.h"
#include "timedelta.h"

#ifndef timersub
#define timersub(a, b, result) \
  do { \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
    if ((result)->tv_usec < 0) { \
      --(result)->tv_sec; \
      (result)->tv_usec += 1000000; \
    } \
  } while (0)
#endif

#ifdef WIN32
int gettimeofday(struct timeval *tv, void *tz)
{
  DWORD ticks = GetTickCount();

  assert(tv != NULL);
  if (tv == NULL)
    return -1;

  tv->tv_sec = ticks / 1000;
  tv->tv_usec = ticks % 1000;
  return 0;
}
#endif /* def WIN32 */

void initTimeDelta(struct timeval *delta, int secsFromNow)
{
  assert(delta != NULL);
  if (delta != NULL)
  {
    gettimeofday(delta, NULL);
    delta->tv_sec += secsFromNow;
  }
}

void adjustTimeDelta(struct timeval *tv, const struct timeval *targetTime)
{
  struct timeval now;

  assert(tv != NULL);
  if (tv == NULL)
    return;
  assert(targetTime != NULL);
  if (targetTime == NULL)
    return;

  gettimeofday(&now, NULL);
  if (timercmp(targetTime, &now, >))
    timersub(targetTime, &now, tv);
  else
    timerclear(tv);
}
