#ifndef __TIME_DELTA_H__
#define __TIME_DELTA_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Initializes delta for the time now + secsFromNow.  Assumes delta is not NULL.
 */
void initTimeDelta(struct timeval *delta, int secsFromNow);

/* Adjusts the timeval tv to be the time left until targetTime is reached.
 * Assumes tv and targetTime are not NULL.
 */
void adjustTimeDelta(struct timeval *tv, const struct timeval *targetTime);

#ifdef WIN32
/* Windows function that sets tv with a relative time--it has no "absolute"
 * meaning, but if called twice and the results of the two calls are
 * differenced with timersub, the resulting value will represent the time
 * elapsed.  Useful for a platform-independent select relying on struct
 * timeval, where the number of seconds from a fixed time in the past is not
 * relevant.
 * Assumes tv is not NULL; tz is ignored.  Assumes that the time between two
 * calls is relatively short, e.g. within the same day or so.
 * Returns 0 on success, -1 on error.
 */
int gettimeofday(struct timeval *tv, void *tz);
#endif

#ifdef __cplusplus
}
#endif

#endif /* ndef __TIME_DELTA_H__ */
